import get from 'lodash/get';
import set from 'lodash/set';
import resolveConfig from 'tailwindcss/resolveConfig';
import invariant from 'tiny-invariant';
import Colors from 'color-name';

const VALID_HEX = /^#([\da-f]{3}|[\da-f]{6})([\da-f]{2})?/i;
const VALID_TW_COLOR_CLASS = /^[a-z]+-\d{2,3}(?![\w-]+)/i;
const VALID_COLOR_FORM = `${VALID_HEX.source}|${VALID_TW_COLOR_CLASS.source}`;
const VALID_ALPHA = /\/(?=(\b([1-9]|[1-9]\d|100)\b))/i;
const isParsableString = (value) => typeof value === "string" &&
    // No need to parse these as chart.js can readily accept it
    !/rgba?|hsla?|transparent/i.test(value);
const isValidArray = (value) => Array.isArray(value) && value.every((v) => typeof v === "string");
const isHex = (value) => VALID_HEX.test(value);
const isNamedColor = (value) => value in Colors;
/**
 * Checks first if the color is in a valid form, then
 * checks if it has a valid `alpha` color channel.
 */
const hasValidAlpha = (value) => 
// TODO There should be room for improvement here
(isNamedColor(value.split("/")[0]) && VALID_ALPHA.test(value)) || // named colors
    new RegExp(`(?:${VALID_COLOR_FORM})${VALID_ALPHA.source}`).test(value);

// Color utilities taken from the CSS framework tailwindcss, which is
// Copyright (c) Tailwind Labs, Inc. (https://tailwindcss.com/)
// and used under the terms of the MIT license
// @see https://github.com/tailwindlabs/tailwindcss/blob/main/LICENSE
const HEX = /^#([\da-f]{2})([\da-f]{2})([\da-f]{2})([\da-f]{2})?$/i;
const SHORT_HEX = /^#([\da-f])([\da-f])([\da-f])([\da-f])?$/i;
// @see https://github.com/tailwindlabs/tailwindcss/blob/main/src/util/flattenColorPalette.js
const flattenColorPalette = (colors) => {
    const result = {};
    Object.entries(colors).forEach(([color, value]) => {
        if (typeof value === "string")
            result[color] = value;
        else {
            const nestedColors = flattenColorPalette(value);
            Object.entries(nestedColors).forEach(([number, hex]) => {
                result[`${color}${number === "DEFAULT" ? "" : `-${number}`}`] = hex;
            });
        }
    });
    return result;
};
// @see https://github.com/tailwindlabs/tailwindcss/blob/main/src/util/color.js
function parseColor(value) {
    value = value.trim();
    if (value === "transparent") {
        return { mode: "rgb", values: ["0", "0", "0"], alpha: "0" };
    }
    if (isNamedColor(value)) {
        return {
            mode: "rgb",
            values: Colors[value].map((v) => v.toString()),
        };
    }
    // We already filter out and validate before even trying
    // to parse, so this should now always match a hex
    const hex = HEX.exec(
    // eslint-disable-next-line @typescript-eslint/max-params
    value.replace(SHORT_HEX, (_, r, g, b, a) => ["#", r, r, g, g, b, b, a ? a + a : ""].join("")));
    invariant(hex, `Invalid value: ${value}`);
    return {
        mode: "rgb",
        values: [
            Number.parseInt(hex[1], 16),
            Number.parseInt(hex[2], 16),
            Number.parseInt(hex[3], 16),
        ].map((v) => v.toString()),
        alpha: hex[4] ? (Number.parseInt(hex[4], 16) / 255).toString() : undefined,
    };
}
function formatColor({ mode, values, alpha }) {
    return `${mode}(${values.join(" ")}${alpha ? ` / ${alpha}` : ""})`;
}

class TailwindColorsParser {
    constructor(config) {
        const colors = Object.assign({}, resolveConfig(config).theme.colors);
        invariant(colors, "TailwindCSS theme colors is undefined!");
        this.colorPalette = flattenColorPalette(colors);
        this.config = config;
    }
    parse(value) {
        var _a, _b;
        if (Array.isArray(value)) {
            return value.map((v) => this.parse(v));
        }
        if (!isParsableString(value))
            return value;
        if (hasValidAlpha(value)) {
            const [color, alpha] = value.split("/");
            return formatColor(Object.assign(Object.assign({}, parseColor((_a = this.colorPalette[color.trim()]) !== null && _a !== void 0 ? _a : color)), { alpha: Number.parseInt(alpha, 10) / 100 }));
        }
        return (_b = this.colorPalette[value.trim()]) !== null && _b !== void 0 ? _b : formatColor(parseColor(value));
    }
    /**
     * Checks if a given color/value is valid, and
     * whether it should to be parsed.
     */
    isParsable(value, { strict = false, hex, named } = {}) {
        if (!value)
            return false;
        if (!strict) {
            // Since some colors are stored in arrays, assuming all values
            // in the array are strings, the array itself is valid
            if (isValidArray(value))
                return true;
            if (!isParsableString(value))
                return false;
            if (hex) {
                return isHex(value);
            }
            if (named) {
                return isNamedColor(value);
            }
            // Ignore hex and named colors without a valid alpha
            return (Object.hasOwn(this.colorPalette, value) || hasValidAlpha(value));
        }
        // Strictly from the specified config
        return (isParsableString(value) && Object.hasOwn(this.colorPalette, value));
    }
}

const parsableOptions = [
    "color",
    "borderColor",
    "backgroundColor",
    "hoverBorderColor",
    "hoverBackgroundColor",
    "pointBorderColor",
    "pointBackgroundColor",
    "pointHoverBorderColor",
    "pointHoverBackgroundColor",
    "fill.above",
    "fill.below",
];
const twColorsPlugin = (tailwindConfig, defaults = {}) => {
    const parser = new TailwindColorsParser(tailwindConfig);
    return {
        id: "tailwindcss-colors",
        beforeLayout: (chart) => {
            parsableOptions.forEach((parsableOpt) => {
                var _a;
                const chartDefaultColor = get(chart.options, parsableOpt);
                const defaultOptColor = (_a = defaults[parsableOpt]) !== null && _a !== void 0 ? _a : chartDefaultColor;
                if (parser.isParsable(defaultOptColor)) {
                    set(chart.options, parsableOpt, parser.parse(defaultOptColor));
                }
                chart.config.data.datasets.forEach((dataset) => {
                    const color = get(dataset, parsableOpt, defaultOptColor);
                    if (parser.isParsable(color)) {
                        set(dataset, parsableOpt, parser.parse(color));
                    }
                });
            });
        },
        // Handle scriptable options
        beforeDatasetDraw: (chart, args) => {
            parsableOptions.forEach((parsableOpt) => {
                var _a;
                const chartOptColor = get(chart.options, parsableOpt);
                const defaultOptColor = (_a = defaults[parsableOpt]) !== null && _a !== void 0 ? _a : chartOptColor;
                const metaDataset = args.meta.dataset;
                if (!metaDataset)
                    return;
                const metaDatasetOptionsColor = get(metaDataset.options, parsableOpt, defaultOptColor);
                if (parser.isParsable(metaDatasetOptionsColor)) {
                    set(metaDataset.options, parsableOpt, parser.parse(metaDatasetOptionsColor));
                }
                const currentDataset = chart.data.datasets[args.index];
                currentDataset.data.forEach((_, index) => {
                    const currentElement = args.meta.data[index];
                    const resolvedColor = get(currentElement.options, parsableOpt, defaultOptColor);
                    if (parser.isParsable(resolvedColor)) {
                        set(currentElement.options, parsableOpt, parser.parse(resolvedColor));
                    }
                });
            });
        },
    };
};

export { twColorsPlugin as default };
//# sourceMappingURL=index.mjs.map
